package LNG
  package units
    package JTvalve
      model BaseValve
        connectors.BaseConnector inlet;
        connectors.BaseConnector outlet;
        parameter Real Cv;
        input Real x;
        
      equation
        outlet.H = -inlet.H;
        outlet.m = -x*Cv*(inlet.P-outlet.P)^0.5;
        
      end BaseValve;
      
      model M
        extends BaseValve(Cv = 3.4);
        import LNG.functions.methane.*;
      equation
        outlet.T = Tsat(outlet.P);
      end M;
      
      model E
        extends BaseValve(Cv = 8.4);
        import LNG.functions.ethane.*;
      equation
        outlet.T = Tsat(outlet.P);
      end E;
      
      model P
        extends BaseValve(Cv = 13);
        import LNG.functions.propane.*;
      equation
        outlet.T = Tsat(outlet.P);
      end P;

    end JTvalve;


package compressor

  model BaseComp
    // inlet and outlet
    connectors.BaseConnector inlet;
    connectors.BaseConnector outlet;
    // Variables from functions
    types.HeatCapacity cp_s;
    types.HeatCapacity cp_d;
    types.Compressibility Z_s;
    // constants and coefficients
    parameter Real R = 8.314;
    parameter Real g = 9.81;
    parameter types.MolarMass Mm;
    // efficieny
    parameter Real e1;
    parameter Real e2;
    parameter Real e3;
    parameter Real e4;
    // compressor curves
    parameter Real C1;
    parameter Real C2;
    parameter Real C3;
    parameter Real p;
    parameter Real q;
    //variables
    Real k;
    Real gamma;
    Real nu;
    Real h_p;
    Real h_p_s;
    Real q_suc;
    Real Wcomp;
    input Real N_M;
  equation
//compressor equations
        outlet.T / inlet.T = (outlet.P / inlet.P) ^ (1 / k);
    k = nu*(gamma/(gamma-1));
    gamma = 1/2 * ((cp_s/(cp_s-R))+(cp_d/(cp_d-R)));
    h_p = 100* (k*Z_s*R/(g*Mm))*(outlet.T-inlet.T);
    
    nu = (e1*h_p_s+e2-2*10^(e3*h_p_s-e4));
    h_p_s = h_p/(N_M^p);
    q_suc/(N_M^q) = (C1*h_p_s-C2)/(h_p_s-C3);
    
    1000*q_suc = inlet.m*R*inlet.T/inlet.P;
//other equations
        inlet.m = -outlet.m;
    -Wcomp = outlet.H+inlet.H;
    
  end BaseComp;
  
  model M
    import LNG.functions.methane.*;
    extends BaseComp(
      
      e1 = 0.0251571,
      e2 = -0.074,
      e3 = 0.138053,
      e4 = 6.5963,
      C1 = 0.27146,
      C2 = 11.765,
      C3 =45.040,
      p = 2.11,
      q = 1.79,
      Mm = 16.04);
      
    equation
      
      cp_s = heatCapacity(inlet.T);
      cp_d = heatCapacity(outlet.T);
      Z_s = compressibility(T = inlet.T, P = inlet.P, Z = Z_s);
      
      outlet.H*100000 = (Hvap(outlet.P)+0.5*(cp_d+heatCapacity(Tsat(outlet.P)))*(outlet.T-Tsat(outlet.P))*100)*outlet.m;
      
  end M;

  
  model E
    import LNG.functions.ethane.*;
    extends BaseComp(
      e1 = 0.03448682,
      e2 = -0.074,
      e3 = 0.18925109,
      e4 = 6.5963,
      C1 = 1.9928,
      C2 = 63.00,
      C3 =32.855,
      p = 2.11,
      q = 1.79,
      Mm = 30.07);
    
    equation
      
      cp_s = heatCapacity(inlet.T);
      cp_d = heatCapacity(outlet.T);
      Z_s = compressibility(T = inlet.T, P = inlet.P, Z = Z_s);
      
      outlet.H*100000 = (Hvap(outlet.P)+0.5*(cp_d+heatCapacity(Tsat(outlet.P)))*(outlet.T-Tsat(outlet.P))*100)*outlet.m;
      
  end E;
  
  model P
    import LNG.functions.propane.*;
    extends BaseComp(
      e1 = 0.061733,
      e2 = -0.074,
      e3 = 0.338767,
      e4 = 6.5963,
      C1 = 1.9928,
      C2 = 35.1962,
      C3 = 18.3546,
      p = 2.11,
      q = 1.79,
      Mm = 44.1);
    
    equation
      
      cp_s = heatCapacity(inlet.T);
      cp_d = heatCapacity(outlet.T);
      Z_s = compressibility(T = inlet.T, P = inlet.P, Z = Z_s);
      
      outlet.H*100000 = (Hvap(outlet.P)+0.5*(cp_d+heatCapacity(Tsat(outlet.P)))*(outlet.T-Tsat(outlet.P))*100)*outlet.m;
      
  end P;


end compressor;




package condenser
  model BaseCondenser
    connectors.BaseConnector inlet;
    connectors.BaseConnector outlet;
    parameter Real Exp;
    Real alpha;
    Real Tcond;
    Real Tambient;
    parameter Real cp_water;
    
    Real UA;
    Real dt1;
    Real dt2;
    Real dtlm;
    Real Qcond;
    types.HeatCapacity cp_d;
    input Real mcond;
  
  equation
    UA = alpha*Exp;
    dt1 = Tcond -inlet.T;
    dt2 = Tambient -outlet.T;
    dtlm = (dt1-dt2)/(Modelica.Math.log(dt1/dt2));
    Qcond = dtlm*UA;
    Qcond = mcond*cp_water*(Tambient-Tcond);
//Balance equations
    inlet.m = -outlet.m;
    inlet.P = outlet.P;
  end BaseCondenser;


  
  model M
    extends BaseCondenser(Exp = 0.27001);
    import LNG.functions.methane.*;
    
    equation
      cp_d = heatCapacity(outlet.T);
      outlet.H*100000 = (Hvap(outlet.P)+0.5*(cp_d+heatCapacity(Tsat(outlet.P)))*(outlet.T-Tsat(outlet.P))*100)*outlet.m;
      - Qcond = outlet.H+inlet.H;
  end M;
  
  model E
    extends BaseCondenser(Exp = 0.45);
    import LNG.functions.ethane.*;
    
    equation
    cp_d = heatCapacity(outlet.T);
    outlet.H*100000 = (Hvap(outlet.P)+0.5*(cp_d+heatCapacity(Tsat(outlet.P)))*(outlet.T-Tsat(outlet.P))*100)*outlet.m;
    -Qcond = outlet.H +inlet.H;

  end E;
  
  model P
    extends BaseCondenser(Exp = 5.1);
    import LNG.functions.propane.*;
    
    equation
    cp_d = heatCapacity(outlet.T);
    
    outlet.T = Tsat(outlet.P);
    outlet.H*100000 = Hliq(outlet.P)*outlet.m;
    -Qcond = outlet.H +inlet.H;

  end P;
end condenser;



  
  package HX
    model HexBase
      connectors.BaseConnector inlet;
      connectors.BaseConnector outlet;
      connectors.LNG inlet_LNG;
      connectors.LNG outlet_LNG;
      parameter Real UA;
      Real dtlm;
      Real dt1;
      Real dt2;
      Real Qhex;
      Real m_LNG;
    equation
// Heat transfer equations
      dt1 = inlet_LNG.T - outlet.T;
      dt2 = outlet_LNG.T - inlet.T;
      dtlm = (dt1-dt2)/Modelica.Math.log(dt1/dt2);
      Qhex = UA * dtlm;
      Qhex = outlet_LNG.H+inlet_LNG.H;
// Balance for cold side
        inlet.P = outlet.P;
      outlet.m = -inlet.m;
//Cold side is only in liquid phase
        inlet.T = outlet.T;
    end HexBase;

  model PL
    import LNG.functions.propane.*;
    extends HexBase(UA = 0.329916759332703);
    
    equation
      -100000*outlet_LNG.H = (4522.5*outlet_LNG.T-13837)*m_LNG;
//Balance equations cold side
      100000*outlet.H = Hvap(outlet.P)*outlet.m;
    
  end PL;
  
  model EL
    import LNG.functions.ethane.*;
    extends HexBase(UA = 0.892593986669065);
    //Enthalpy functions

    function heavi
      input Real x;
      output Real temp;
    algorithm
      temp := Modelica.Math.exp(200*x)/(1+Modelica.Math.exp(200*x));
    end heavi;
    function f_H
      input Real T;
      output Real temp;
    algorithm
      temp := heavi(1.855-T)*( 9912.5*T-27757) + heavi(T-1.855)*(11131.0*T-26943);
    end f_H;
    
    equation
      -100000*outlet_LNG.H = f_H(outlet_LNG.T)*m_LNG;
//Balance equations cold side
      100000*outlet.H = Hvap(outlet.P)*outlet.m;
    
  end EL;
  
  model ML
    import LNG.functions.methane.*;
    extends HexBase(UA = 0.538401346748178);
    
    equation
      -100000*outlet_LNG.H = (5664.6*outlet_LNG.T-20725)*m_LNG;
//Balance equations cold side
      100000*outlet.H = Hvap(outlet.P)*outlet.m;
    
  end ML;
  
  end HX;

package receiver

  model oldRec
    import LNG.functions.methane.*;
    connectors.BaseConnector inlet;
    connectors.BaseConnector outlet;
    parameter Real mRec;
    Real Hrec;
   equation
      //balance
        inlet.m = -outlet.m;
      //outlet conditions (liquid)
        -outlet.H = Hrec * outlet.m;
    outlet.H*100000 = Hliq(outlet.P)*outlet.m;
    outlet.T = Tsat(outlet.P);
    -der(Hrec) = (inlet.H+outlet.H)/mRec; 
  end oldRec;
  
  
  model Rec
    connectors.BaseConnector inlet;
    connectors.BaseConnector outlet;
    types.HeatCapacity cp_d;
    parameter Real mRec;
    Real Hrec;
   equation
 
    inlet.m = -outlet.m;
    inlet.P = outlet.P;
    -outlet.H = Hrec*outlet.m;
    -der(Hrec) = (inlet.H+outlet.H)/mRec; 
  end Rec;
  
  model M
    extends Rec();
    import LNG.functions.methane.*;
  
  equation
    cp_d = heatCapacity(outlet.T);
    outlet.H*100000 = (Hvap(outlet.P)+0.5*(cp_d+heatCapacity(Tsat(outlet.P)))*(outlet.T-Tsat(outlet.P))*100)*outlet.m;
  
  end M;
  
  model E
    extends Rec();
    import LNG.functions.ethane.*;
  
  equation
    cp_d = heatCapacity(outlet.T);
    outlet.H*100000 = (Hvap(outlet.P)+0.5*(cp_d+heatCapacity(Tsat(outlet.P)))*(outlet.T-Tsat(outlet.P))*100)*outlet.m;
  
  end E;

  model RecP
    import LNG.functions.propane.*;
    connectors.BaseConnector inlet;
    connectors.BaseConnector outlet;
    parameter Real mRec;
    Real Trec;
  equation
  
   inlet.m = -outlet.m;
   der(Trec) = inlet.m/mRec * (inlet.T-Trec);
   outlet.T = Trec;
   outlet.T = Tsat(outlet.P);
   outlet.H*100000 = Hliq(outlet.P)*outlet.m;   
  end RecP;

end receiver;

package hex
  model BaseHex
    connectors.BaseConnector hotin;
    connectors.BaseConnector hotout;
    connectors.BaseConnector coldin;
    connectors.BaseConnector coldout;
    parameter Real UA;
    Real dtlm;
    Real dt1;
    Real dt2;
    //TEMP parameter
    Real Qhex;
    
  equation
// Heat transfer equations
        dt1 = hotin.T - coldout.T;
    dt2 = hotout.T - coldin.T;
    dtlm = (dt1-dt2)/Modelica.Math.log(dt1/dt2);
    Qhex = UA * dtlm;
// Balance for cold side
    hotin.P = hotout.P;
    hotin.m = -hotout.m;
    coldin.P = coldout.P;
    coldin.m = -coldout.m;
//Cold side only in liquid phase
        coldin.T = coldout.T;
  end BaseHex;

  
  model PM
  extends BaseHex(UA = 0.294778127170467);
    types.HeatCapacity cp_d;
  // hot side is methane
  import LNG.functions.methane.*;
  equation
    cp_d = heatCapacity(hotout.T);
    hotout.H*100000 = (Hvap(hotout.P)+0.5*(cp_d+heatCapacity(Tsat(hotout.P)))*(hotout.T- Tsat(hotout.P))*100)*hotout.m;
    Qhex = hotout.H+hotin.H; //cold side
    -Qhex = coldout.H +coldin.H;
  end PM;

  model PE
  extends BaseHex(UA = 2.674439684513716);
    types.HeatCapacity cp_d;
  // hot side is ethane
  import LNG.functions.ethane.*;
  equation
    cp_d = heatCapacity(hotout.T);
    hotout.H*100000 = Hliq(hotout.P)*hotout.m;
    hotout.T = Tsat(hotout.P);
    Qhex = hotout.H+hotin.H;
//coldside
    -Qhex = coldout.H +coldin.H;
  end PE;
  
 
  model EM
  extends BaseHex(UA = 0.829808335718488);
    types.HeatCapacity cp_d;
  // hot side is methane
  import LNG.functions.methane.*;
  equation
//hotside
        cp_d = heatCapacity(hotout.T);
    hotout.H*100000 = Hliq(hotout.P)*hotout.m;
    hotout.T = Tsat(hotout.P);
    Qhex = hotout.H+hotin.H;
//coldside
        -Qhex = coldout.H + coldin.H;
  end EM;


   model EM_old
  extends BaseHex(UA = 0.829808335718488);
    types.HeatCapacity cp_d;
  // hot side is methane
  import LNG.functions.methane.*;
  
  equation
    cp_d = heatCapacity(hotout.T);
    hotout.H*100000 = (Hliq(hotout.P)+0.5*(cp_d+heatCapacity(Tsat(hotout.P)))*(hotout.T- Tsat(hotout.P))*100)*hotout.m;
// finds temperature from enthalpy relation
    Qhex = hotout.H+hotin.H; 
  end EM_old;
end hex;


package sources
  model source
    import LNG.functions.methane.*;
    connectors.BaseConnector outlet;
    parameter Real m_init = 1;
    parameter Real P_init = 1;
    parameter Real T_init = 1;
    parameter Real H_init = 1;
    equation
      outlet.m = m_init;
      outlet.P = P_init;
      outlet.T = T_init;
      outlet.H = H_init;
  end source;
  
  model sourceE
    import LNG.functions.methane.*;
    connectors.BaseConnector outlet;
    parameter Real m_init = 1;
    parameter Real P_init = 1;
    parameter Real T_init = 1;
    //parameter Real H_init = 1;
    equation
      outlet.m = m_init;
      outlet.P = P_init;
      outlet.T = T_init;
//outlet.H = H_init;
  end sourceE;

 
  model sink
  
    connectors.BaseConnector inlet;
  end sink;
  
  model LNGsource
    connectors.LNG outlet;
    Real T_init;
    Real H_init;
    equation
      outlet.H = H_init;
      outlet.T = T_init;
  end LNGsource;


  
  model LNGsink
    connectors.LNG inlet;
  end LNGsink;

  end sources;

  
  package tank
   model tank
      connectors.BaseConnector inlet;
      connectors.BaseConnector outlet;
      
      
      equation
      inlet.T = outlet.T;
      inlet.P = outlet.P;
      inlet.H = -outlet.H;
      outlet.m = -inlet.m;
   end tank;

  model M
      //extends tank();
      connectors.BaseConnector inlet;
      connectors.BaseConnector outlet;
      parameter Real mRec;
      Real Trec;
      import LNG.functions.methane.*;
      equation
      inlet.m = -outlet.m;
      der(Trec) = inlet.m/mRec * (inlet.T-Trec);
      outlet.T = Trec;
      outlet.T = Tsat(outlet.P);
      outlet.H*100000 = Hvap(outlet.P)*outlet.m;
      
  end M;
  end tank;
  


 



  end units;






  package types
    type Temperature = Real;
    type Pressure = Real;
    type Molarflow = Modelica.SIunits.MolarFlowRate(nominal = 3);
    type Enthalpy = Real(nominal = 1);
    type HeatCapacity = Real(nominal = 50, unit = "J/mol.K");
    type Compressibility = Real(nominal = 1);
    type Work = Modelica.SIunits.Work(nominal = 1);
    type Heat = Modelica.SIunits.Heat(nominal = 1);
    type MolarMass = Modelica.SIunits.MolarMass();
  end types;


  package connectors
    connector BaseConnector
      
      types.Pressure P;
      types.Temperature T;
       
      flow types.Molarflow m;
      flow types.Enthalpy H;
    end BaseConnector;



    connector LNG 
      types.Temperature T;
      flow types.Molarflow H;
    end LNG;
    connector testing
      Real variable;
    end testing;
  end connectors;



  package functions
    package methane
      function heatCapacity
        input Real T;
        output Real cp;
      protected
        parameter Real R = 8.31446;
        parameter Real c1 = 1.702;
        parameter Real c2 = 0.0090819;
        parameter Real c3 = -2.164e-6;
      algorithm
        cp := (c1 +c2*T*100+c3*(T*100)^2)*R;  
      end heatCapacity;
      function compressibility
        input Real T;
        input Real P;
        input Real Z;
        output Real Z_out;
      protected
        parameter Real Tc = 1.90;
        parameter Real Pc = 4.60551724137931;
        parameter Real A1 = 0.3265;
        parameter Real A2 = -1.0700;
        parameter Real A3 = -0.5339;
        parameter Real A4 = 0.01569;
        parameter Real A5 = -0.05165;
        parameter Real A6 = 0.5475;
        parameter Real A7 = -0.7361;
        parameter Real A8 = 0.1844;
        parameter Real A9 = 0.1056;
        parameter Real A10 = 0.6134;
        parameter Real A11 = 0.7210;
        Real Tpr = T/Tc;
        Real Ppr = P/Pc;
        Real tmp = (0.27*Ppr/(Z*Tpr));
      algorithm
        Z_out := 1+(A1+A2/Tpr+A3/Tpr^3+A4/Tpr^4+A5/Tpr^5)*tmp +(A6+A7/Tpr+A8/Tpr^2)*tmp^2 - (A7/Tpr+A8/Tpr^2)*A9*tmp^5 + A10*(1+A11*tmp^2)*(tmp^2/Tpr^3)*Modelica.Math.exp(-A11*tmp^2);
      end compressibility;

      function Hvap
        input Real P;
        output Real H;
      algorithm
        H := -59.04*P^4 + 518.47*P^3 - 1650.9*P^2 + 2117.7*P - 6622.6;
      end Hvap;
      function Hliq
        input Real P;
        output Real H;
      algorithm
        H := -7.6457*P^6 + 138.42*P^5 - 943.33*P^4 + 3138.6*P^3 - 5488.7*P^2 + 5974.2*P - 15187;
      end Hliq;
      function Tsat
        input Real P;
        output Real T;
      algorithm
        T := 0.13110*Modelica.Math.log10(P)^2 + 0.51530*Modelica.Math.log10(P) + 1.4966;
      end Tsat;
      
    end methane;
  package ethane
      function heatCapacity
        input Real T;
        output Real cp;
      protected
        parameter Real R = 8.31446;
        parameter Real c1 = 1.131;
        parameter Real c2 = 19.225e-3;
        parameter Real c3 = -5.561e-6;
      algorithm
        cp := (c1 +c2*T*100+c3*(T*100)^2)*R;  
      end heatCapacity;
function compressibility
  input Real T;
  input Real P;
  input Real Z;
  output Real Z_out;
protected
  parameter Real Tc = 3.05;
  parameter Real Pc = 4.88137931034483;
  parameter Real A1 = 0.3265;
  parameter Real A2 = -1.0700;
  parameter Real A3 = -0.5339;
  parameter Real A4 = 0.01569;
  parameter Real A5 = -0.05165;
  parameter Real A6 = 0.5475;
  parameter Real A7 = -0.7361;
  parameter Real A8 = 0.1844;
  parameter Real A9 = 0.1056;
  parameter Real A10 = 0.6134;
  parameter Real A11 = 0.7210;
  Real Tpr = T/Tc;
  Real Ppr = P/Pc;
  Real tmp = (0.27*Ppr/(Z*Tpr));
algorithm
  Z_out := 1+(A1+A2/Tpr+A3/Tpr^3+A4/Tpr^4+A5/Tpr^5)*tmp +(A6+A7/Tpr+A8/Tpr^2)*tmp^2 - (A7/Tpr+A8/Tpr^2)*A9*tmp^5 + A10*(1+A11*tmp^2)*(tmp^2/Tpr^3)*Modelica.Math.exp(-A11*tmp^2);
end compressibility;

      function Hvap
        input Real P;
        output Real H;
      algorithm
        H := -25.103*P^6 + 356.06*P^5 - 1973*P^4 + 5412.9*P^3 - 7884.9*P^2 + 6200.8*P + 5906;
      end Hvap;
      
      function Hliq
        input Real P;
        output Real H;
      algorithm
        H := -18.124*P^6 + 319.1*P^5 - 2146.5*P^4 +7061.9*P^3 - 12132*P^2 + 12605*P - 9447.4;
      end Hliq;
      function Tsat
        input Real P;
        output Real T;
      algorithm
        T := 0.21080*Modelica.Math.log10(P)^2 + 0.77135*Modelica.Math.log10(P) + 2.4141;
      end Tsat;
      
    end ethane;


    package propane
      function heatCapacity
        input Real T;
        output Real cp;
      protected
        parameter Real R = 8.31446;
        parameter Real c1 = 1.213;
        parameter Real c2 = 28.785e-3;
        parameter Real c3 = -8.824e-6;
      algorithm
        cp := (c1 +c2*T*100+c3*(T*100)^2)*R;  
      end heatCapacity;
function compressibility
  input Real T;
  input Real P;
  input Real Z;
  output Real Z_out;
protected
  parameter Real Tc = 3.70;
  parameter Real Pc = 4.25034482758621;
  parameter Real A1 = 0.3265;
  parameter Real A2 = -1.0700;
  parameter Real A3 = -0.5339;
  parameter Real A4 = 0.01569;
  parameter Real A5 = -0.05165;
  parameter Real A6 = 0.5475;
  parameter Real A7 = -0.7361;
  parameter Real A8 = 0.1844;
  parameter Real A9 = 0.1056;
  parameter Real A10 = 0.6134;
  parameter Real A11 = 0.7210;
  Real Tpr = T/Tc;
  Real Ppr = P/Pc;
  Real tmp = (0.27*Ppr/(Z*Tpr));
algorithm
  Z_out := 1+(A1+A2/Tpr+A3/Tpr^3+A4/Tpr^4+A5/Tpr^5)*tmp +(A6+A7/Tpr+A8/Tpr^2)*tmp^2 - (A7/Tpr+A8/Tpr^2)*A9*tmp^5 + A10*(1+A11*tmp^2)*(tmp^2/Tpr^3)*Modelica.Math.exp(-A11*tmp^2);
end compressibility;

      function Hvap
        input Real P;
        output Real H;
      algorithm
        H := -56.405*P^6 + 721.52*P^5 - 3644.6*P^4 + 9271.9*P^3 - 12980*P^2 + 11005*P + 9036.9;
      end Hvap;
      function Hliq
        input Real P;
        output Real H;
      algorithm
        H := 4.8879*P^6 + 50.621*P^5 - 970.54*P^4 + 4838.4*P^3 - 11110*P^2 + 15991*P - 9992.7;
      end Hliq;
      function Tsat
        input Real P;
        output Real T;
      algorithm
        T := 0.24817*Modelica.Math.log10(P)^2 + 0.93582*Modelica.Math.log10(P) + 3.0053;
      end Tsat;
      
    end propane;
function LNG_H
        input Real T;
        input Real m;
        output Real H;
      algorithm
        H := ((4348.7*T-13430)*m)/100000;
      end LNG_H;
    
  end functions;

model Process
  //parameters
   Real LNGtemp;
   Real LNGflow;
   Real LNGH = -functions.LNG_H(T = LNGtemp,m = LNGflow);
   Real Tamb;
   Real alphaM;
   Real alphaE;
   Real alphaP;
   parameter Real cp_w = 75.3;
  
   
  //Methane loop
  units.tank.tank tankM(
   outlet.m(start = -3.26));
 //compressor
  units.compressor.M compM(
    k(start = 2.66),
    Z_s(start = 0.928219),
    h_p_s(start = 40),
    outlet.T(start = 3.56),
    N_M(start =0.918207),
    inlet.T(start = 1.1827),
    inlet.P(start = 0.17637),
    inlet.m(start = 3.26069),
    inlet.H(start = -0.2053));
    //condenser
  units.condenser.M condM(
    alpha = alphaM,
    Tambient = Tamb,
    cp_water = cp_w,
    outlet.T(start = 2.87141),
    dt1(start = 0.5),
    dt2(start = 0.05),
    Tcond(start = 2.83));
    //receiver
  units.receiver.M receiverM(
    mRec = 500,
    outlet.P(start = 3.34));
    //heat exchangers
  units.hex.PM hexPM(
    hotout.T(start = 2.33),
    hotin.T(start= 2.87),
    coldin.T(start = 2.29681),
    coldout.T(start = 2.29681),
    dt2(start = 0.05),
    dt1(start = 0.5));
  
  units.hex.EM hexEM(
   hotout.T(start = 1.81),
   hotout.P(start = 3.34),
   coldin.T(start = 1.75),
   coldout.T(start = 1.75),
   dt1(start = 0.5),
   dt2(start = 0.05));
  //JT valve
  units.JTvalve.M valveM();
  //LNG methane exchanger
  units.HX.ML hexML(m_LNG = LNGflow,outlet_LNG.T(start = 1.23),dt1(start = 0.5),dt2(start = 0.05),outlet.T(start = 1.17),outlet.P(start = 0.176));
  
  
  //Ethane loop
  units.tank.tank tankE(
    outlet.m(start = -3.67));
  //compressor
  units.compressor.E compE(
    k(start = 4.2),
    Z_s(start = 0.979),
    h_p_s(start = 26),
    outlet.T(start = 3.51),
    N_M(start =0.88),
    inlet.T(start = 1.75),
    inlet.P(start = 0.044),
    inlet.m(start = 3.67),
    inlet.H(start = 0.226));
    //condenser
  units.condenser.E condE(
    alpha = alphaE,
    Tambient = Tamb,
    cp_water = cp_w,
    outlet.T(start = 2.87141),
    Tcond(start = 2.83));
  //receiver
  units.receiver.E receiverE(
    mRec = 350,
    outlet.P(start = 0.79));
    //heat exchangers
  units.hex.PE hexPE(
    hotout.T(start = 2.336),
    hotin.T(start= 2.877),
    coldin.T(start = 2.29681),
    coldout.T(start = 2.29681),
    dt1(start = 0.5),
    dt2(start = 0.05));
    //JT valve
  units.JTvalve.E valveE();
    //LNG ethane exchanger
  units.HX.EL hexEL(m_LNG = LNGflow,outlet_LNG.T(start = 1.806),dt1(start = 0.5),dt2(start = 0.05),outlet.T(start = 1.75));
  
  
  //propane loop
  units.tank.tank tankP(
    outlet.m(start = -5.37));
  //compressor
  units.compressor.P compP(
    k(start = 4.2),
    Z_s(start = 0.969),
    h_p_s(start = 15),
    outlet.T(start = 3.158),
    N_M(start =0.88),
    inlet.T(start = 2.3),
    inlet.P(start = 0.089),
    inlet.m(start = 5.37),
    inlet.H(start = 0.533));
    //condenser
  units.condenser.P condP(
    alpha = alphaP,
    Tambient = Tamb,
    cp_water = cp_w,
    outlet.T(start = 2.87141),
    inlet.T(start = 3.15),
    outlet.P(start =0.7),
    Tcond(start = 2.83807),
    dt1(start = 0.5),
    dt2(start = 0.05));
  //receiver
  units.receiver.RecP receiverP(
    mRec = 300,
    outlet.P(start = 0.79));
    //JT valve
  units.JTvalve.P valveP();
  //hex
  units.HX.PL hexPL(m_LNG = LNGflow,outlet_LNG.T(start = 2.349),dt1(start = 0.5),dt2(start = 0.05),outlet.T(start = 2.295));
  
  //LNG source and sink
  units.sources.LNGsource LNGsource(T_init = LNGtemp, H_init = LNGH);
  units.sources.LNGsink LNGsink();
  
  equation
  
//methane loop
//tank
    connect(tankM.outlet, compM.inlet);
//Compressor
    connect(compM.outlet, condM.inlet);
//condenser
    connect(condM.outlet, receiverM.inlet);
//receiver
    connect(receiverM.outlet, hexPM.hotin);
//heat exchangers
    connect(hexPM.hotout, hexEM.hotin);
    connect(hexEM.hotout,valveM.inlet);
//valve
    connect(valveM.outlet, hexML.inlet);
//LNG heat exchanger
    connect(hexML.outlet, tankM.inlet);
    
//Ethane loop
//tank
    connect(tankE.outlet, compE.inlet);
//compressor
    connect(compE.outlet, condE.inlet);
//condenser
    connect(condE.outlet, receiverE.inlet);
//receiver
    connect(receiverE.outlet, hexPE.hotin);
//heat exchanger
    connect(hexPE.hotout, valveE.inlet);
//valve
    connect(valveE.outlet, hexEM.coldin);
//heat exchanger
    connect(hexEM.coldout, hexEL.inlet);
//LNG heat exchanger
    connect(hexEL.outlet, tankE.inlet);
    
//Propane loop
  //tank
    connect(tankP.outlet, compP.inlet);
//compressor
    connect(compP.outlet, condP.inlet);
//condenser
  connect(condP.outlet,receiverP.inlet);
//receiver
    connect(receiverP.outlet, valveP.inlet);
//valve
    connect(valveP.outlet, hexPE.coldin);
//Hex
    connect(hexPE.coldout, hexPM.coldin);
    connect(hexPM.coldout,hexPL.inlet);
    connect(hexPL.outlet,tankP.inlet);
 
    connect(LNGsource.outlet,hexPL.inlet_LNG);
    connect(hexPL.outlet_LNG,hexEL.inlet_LNG);
    connect(hexEL.outlet_LNG,hexML.inlet_LNG);
    connect(hexML.outlet_LNG,LNGsink.inlet);

//loop energy
  compM.Wcomp + hexML.Qhex = (-condM.Qcond) + hexEM.Qhex + hexPM.Qhex; 
  compE.Wcomp + hexEL.Qhex + hexEM.Qhex = -condE.Qcond + hexPE.Qhex ;
  compP.Wcomp + hexPL.Qhex + hexPM.Qhex + hexPE.Qhex = -condP.Qcond;
end Process;





model easy
  // Used for PI simulations
   Process process();
   parameter Real T_final = 2.83;
   parameter Real TL_final = 2.9305 ;
   // for LNG test parameter Real TL_final = 2.9805 ;
   parameter Real alphaMfin = 1;
   parameter Real alphaEfin = 1; 
   parameter Real alphaPfin = 1; 
   parameter Real Rate = 0.1;
  
  
   parameter Real u1 = 0.9186759;
   parameter Real u2 = 0.8813871;
   parameter Real u3= 0.8295928;
   parameter Real u4= 0.1219701;
   parameter Real u5= 0.1082355;
   parameter Real u6= 0.9999975;
   
   parameter Real T_set_M = 1.23;
   parameter Real T_set_E = 1.80777;
   parameter Real T_set_P = 2.35067;
   
   parameter Real T_set_M_c = 2.86224;
   parameter Real T_set_E_c = 2.86356;
   parameter Real T_set_P_c = 2.86078;
   
   Real compWork;
   Real Tamb_step ; 
   Real TL_step;
   Real alphaM_step;
   Real alphaE_step;
   
   
   
   //controllers
   Modelica.Blocks.Continuous.PI pi_M(k= 0.1);
   Modelica.Blocks.Continuous.PI pi_E(k= 0.1);
   Modelica.Blocks.Continuous.PI pi_P(k= 0.1);
   
   Modelica.Blocks.Nonlinear.FixedDelay delayM(delayTime = 10);
   Modelica.Blocks.Nonlinear.FixedDelay delayE(delayTime = 10);
   Modelica.Blocks.Nonlinear.FixedDelay delayP(delayTime = 10);

  initial equation
   der(process.receiverM.Hrec) = 0;
   der(process.receiverE.Hrec) = 0;
   der(process.receiverP.Trec) = 0;
   
   process.Tamb = 2.78;
   Tamb_step = process.Tamb;
   
   process.LNGtemp = 2.9305 ;
   TL_step = process.LNGtemp;
   
   
   process.LNGflow = 3.074 ;
   
   process.alphaM = 1 ;
   process.alphaE = 1;
   process.alphaP = 1 ;
   
   alphaM_step = process.alphaM;
   alphaE_step = process.alphaE;

   
  equation
  
    when time >= 75 then
      Tamb_step = T_final;
      alphaM_step = alphaMfin;
      alphaE_step = alphaEfin;
      TL_step = TL_final;
    end when;
    pi_M.u = T_set_M - process.LNGsink.inlet.T;
    pi_E.u = T_set_E - process.hexEL.outlet_LNG.T;
    pi_P.u = T_set_P - process.hexPL.outlet_LNG.T;
  
    delayM.u = pi_M.y;
    delayE.u = pi_E.y;
    delayP.u = pi_P.y;
   
   
    process.compM.N_M = u1 - delayM.y;
    process.compE.N_M = u2 - delayE.y;
    process.compP.N_M = u3 - delayP.y;

    process.condM.mcond = u4;
    process.condE.mcond = u5;
    process.condP.mcond = u6;
  
    der(process.alphaM) = Rate*(alphaM_step-process.alphaM) ;
    der(process.alphaE) = Rate*(alphaE_step-process.alphaE);
    process.alphaP = 1 ;
    der(process.Tamb) = Rate*(Tamb_step-process.Tamb);
    der(process.LNGtemp) = Rate*(TL_step-process.LNGtemp) ;
    process.LNGflow = 3.074 ;
    
    compWork = process.compM.Wcomp+process.compE.Wcomp+process.compP.Wcomp;


end easy;



end LNG;
