import os.path
import csv
# Import numerical libraries
import numpy as N
import matplotlib.pyplot as plt

# Import the needed JModelica.org Python methods
from pymodelica import compile_fmu
from pyfmi import load_fmu
from pyjmi import transfer_optimization_problem, get_files_path
from pyjmi.optimization.casadi_collocation import BlockingFactors
from pyjmi.symbolic_elimination import BLTOptimizationProblem, EliminationOptions


def run_demo(with_plots=True):
    #Lists for importing results from other simulations
    #that are stored in csv files
    timePI = []
    workPI = []
    tempI = []
    compMPI = []
    compEPI = []
    compPPI = []
    myarray = []
    timeR = []
    workR = []
    tempR = []
    compMR = []
    compER = []
    compPR = []

    
    #Importing other data
    with open('exportedVariables.csv', 'rb') as f:
        reader = csv.reader(f)
        for row in reader:
            
            timePI.append(row[0])
            workPI.append(row[1])
            tempI.append(row[2])
            compMPI.append(row[3])
            compEPI.append(row[4])
            compPPI.append(row[5])

    with open('robustdata.csv', 'rb') as f:
        reader = csv.reader(f)
        for row in reader:
            myarray.append(row)
            
    timeR = myarray[0]
    workR = myarray[1]
    tempR = myarray[2]
    compMR = myarray[3]
    compER = myarray[4]
    compPR = myarray[5]
    tempPI = [(float(i)*100-273) for i in tempI]
    
    #Start of actual program
    file_path = os.path.join(get_files_path(), "LNGtest.mop")    
    init_fmu = compile_fmu("LNG.easy", file_path)
    #Simulation model
    init_model = load_fmu(init_fmu)

    
    #Inputs used for initial simulations
    #compM_A = 0.94
    #compE_A = 0.89
    #compP_A = 0.86
    #mcondM_A = 0.1
    #mcondE_A = 0.1
    #mcondP_A = 0.3
    
    #Input for stationary optimal point A
    compM_A = 0.9186759
    compE_A = 0.8813871
    compP_A = 0.8295928
    mcondM_A = 0.1219701
    mcondE_A = 0.1082355
    mcondP_A = 0.9999975
    init_model.set('u1',compM_A)
    init_model.set('u2',compE_A)
    init_model.set('u3',compP_A)
    init_model.set('u4',mcondM_A)
    init_model.set('u5',mcondE_A)
    init_model.set('u6',mcondP_A)

    
    # Solve the initialization problem A using FMI
    init_model.initialize()
    #Save dyanmic states at point A
    [HrecM_A,HrecE_A,TrecP_A] = init_model.get(['process.receiverM.Hrec', 'process.receiverE.Hrec','process.receiverP.Trec'])
    
    #simulation options
    opts = init_model.simulate_options()
    opts['initialize'] = False  
    opts['CVode_options']['atol'] = 1.0e-9
    opts['CVode_options']['rtol'] = 1.0e-7

    #Other random input values
    compM_B = 0.93
    compE_B = 0.87
    compP_B = 0.82
    mcondM_B = 0.05
    mcondE_B = 0.03
    mcondP_B = 0.25
    #Set input B
    init_model.set('u1',compM_B)
    init_model.set('u2',compE_B)
    init_model.set('u3',compP_B)
    init_model.set('u4',mcondM_B)
    init_model.set('u5',mcondE_B)
    init_model.set('u6',mcondP_B)
    #Simulate with input B for initial trajectory
    init_res = init_model.simulate(start_time=0., final_time=750.,options = opts)

    #Optimization problem
    op = transfer_optimization_problem('LNG.opt', file_path)
    
    #Simulation model (model controlled)
    sim_fmu = compile_fmu("LNG.easysim", file_path)
    sim_model = load_fmu(sim_fmu)
    
    #Initial simulation conditions
    sim_model.set('HrecM',HrecM_A)
    sim_model.set('HrecE',HrecE_A)
    sim_model.set('TrecP',TrecP_A)
    #Simulation model options
    opts_S = init_model.simulate_options()
    opts_S['initialize'] = False  
    opts_S['CVode_options']['atol'] = 1.0e-9
    opts_S['CVode_options']['rtol'] = 1.0e-7

    #Discretization options
    sample_time = 15
    horizon = 750
    n_e = horizon/sample_time
    final_t = 150
    num_input = final_t/sample_time

    #set Initial receiver states in OCP
    op.set('HrecM',float(HrecM_A))
    op.set('HrecE',float(HrecE_A))
    op.set('TrecP',float(TrecP_A))
    #options
    opts = op.optimize_options()
    opts['n_e'] = n_e # Number of elements
    opts['init_traj'] = init_res # Initial trajectory
    opts['nominal_traj'] = init_res
    opts['IPOPT_options']['tol'] = 1e-7

    #Blockingfactors
    # Set blocking factors
    factors = {'u1': opts['n_e'] / 1 * [1],
               'u2': opts['n_e'] / 1 * [1],
               'u3': opts['n_e'] / 1 * [1],
               'u4': opts['n_e'] / 1 * [1],
               'u5': opts['n_e'] / 1 * [1],
               'u6': opts['n_e'] / 1 * [1]}
    #Quadric penalties can be used (not used here)
    du_quad_pen = {'u1':10,
                   'u2':10,
                   'u3':10,
                   'u4':10,
                   'u5':10,
                   'u6':10}
    
    bf = BlockingFactors(factors)
    opts['blocking_factors'] = bf #Use blocking factors

    #Lists used for logging simulation model states
    HM = []
    HE = []
    TP = []
    
    compM = []
    compE = []
    compP = []
    compW = []
    
    waterM = []
    waterE = []
    waterP = []
    valveM = []
    valveE = []
    valveP = []
    tempL = []
    Tamb = []
    Te = []
    Press = []
    mm =[]
    hexEL = []
    hexPL = []
    time = []
    time_o = []
    Tamb_o = []

    #Control loop, each iteration is a control step
    for n in range(num_input):
        if n == 0:
            #Initialize sim model and solver
            sim_model.initialize()
            solver = op.prepare_optimization(options=opts)
        #Environment variables can be changed
        if n == 5:
            #sim_model.set('LNGtemp_set',2.9805)
            sim_model.set('Tamb_set',2.83)
        #if n == 8:
           # sim_model.set('Tamb_set',2.7875)
            #sim_model.set('Tamb_set',2.83)
            
        
        #Solve optimization problem
        #Set new time for solver
        solver.collocator.t0 = n*sample_time
        solver.collocator.tf = n*sample_time+horizon
        #Solve problem
        res = solver.optimize()

        #Logging time and ambient temperature used by solver
        Tamb_o.extend(res['process.Tamb'][:4]*100-273)
        time_o.extend(res['time'][:4]+n*sample_time)

        
        #Get opt input from solution in res
        opt_input = res.get_opt_input()

        #Simulate of model using optimal inputs just calculated
        sim_res = sim_model.simulate(start_time= n*sample_time, final_time=(n+1)*sample_time,
                             input=(opt_input), options=opts_S)
        
        #Log changes to dynamic states inputs for plotting 
        HM.extend(sim_res['process.receiverM.Hrec']*10)
        HE.extend(sim_res['process.receiverE.Hrec']*10)
        TP.extend(sim_res['process.receiverP.Trec']*100-273)
        
        compM.extend(sim_res['process.compM.N_M'])
        compE.extend(sim_res['process.compE.N_M'])
        compP.extend(sim_res['process.compP.N_M'])
        compW.extend(sim_res['process.compM.Wcomp']*100+sim_res['process.compE.Wcomp']*100+sim_res['process.compP.Wcomp']*100)
        #Logging random variables
        waterM.extend(sim_res['process.condM.mcond'])
        waterE.extend(sim_res['process.condE.mcond'])
        waterP.extend(sim_res['process.condP.mcond'])
        valveM.extend(sim_res['process.valveM.x'])
        valveE.extend(sim_res['process.valveE.x'])
        valveP.extend(sim_res['process.valveP.x'])
        hexEL.extend(sim_res['process.hexEL.outlet_LNG.T'])
        hexPL.extend(sim_res['process.hexPL.outlet_LNG.T'])
        tempL.extend(sim_res['process.LNGsink.inlet.T']*100-273)
        Tamb.extend(sim_res['process.Tamb']*100-273)
        Te.extend(sim_res['process.condE.outlet.T'])
        Press.extend(sim_res['process.compM.outlet.P'])
        mm.extend(sim_res['process.valveM.outlet.m'])
        time.extend(sim_res['time'])

        #Generate  random noise 
        num1 = N.random.normal(1,0.005)
        num2 = N.random.normal(1,0.005)
        num3 = N.random.normal(1,0.005)
        
        #Extract dynamic staets at the end of the simulation
        #If noise is wanted, multiply with these variables
        HrecM = sim_res['process.receiverM.Hrec'][-1]
        HrecE = sim_res['process.receiverE.Hrec'][-1]
        TrecP = sim_res['process.receiverP.Trec'][-1]

        #Environmental variables are extracted in the same way
        Tambient = sim_res['process.Tamb'][-1]
        LNGtemp = sim_res['process.LNGtemp'][-1]
        LNGflow = sim_res['process.LNGflow'][-1]
        alphaM = sim_res['process.alphaM'][-1]
        alphaE = sim_res['process.alphaE'][-1]
        alphaP = sim_res['process.alphaP'][-1]

        #Set new initial conditions for solver to prepare for next step
        solver.set('HrecM',float(HrecM))
        solver.set('HrecE',float(HrecE))
        solver.set('TrecP',float(TrecP))

        #toggle these off for unknown change (Used for robust comparrison)
        solver.set('Tamb_set',float(Tambient))
        solver.set('Tamb',float(Tambient))
        
        solver.set('LNGtemp_set',float(LNGtemp))
        solver.set('LNGtemp',float(LNGtemp))
        solver.set('LNGflow',float(LNGflow))
        solver.set('alphaM',float(alphaM))
        solver.set('alphaE',float(alphaE))
        solver.set('alphaP',float(alphaP))
        
        #Set warm start
        solver.set_warm_start(True)
        set_warm_start_options(solver)
        



    #Plotting of random variabes logged from simulation model

    plt.figure(1)
    plt.grid(True)
    plt.hold(True)

    plt.subplot(3, 1, 1)
    plt.plot(time, HM)
    plt.ylabel(r'Hrec$_M$ $\frac{J}{mol}$')
    plt.title('Dynamic states')
    plt.subplot(3, 1, 2)
    plt.plot(time, HE)
    plt.grid(True)
    plt.ylabel(r'Hrec$_E$ $\frac{J}{mol}$')
    plt.subplot(3, 1, 3)
    plt.plot(time, TP)
    plt.grid(True)
    plt.ylabel(r'Trec$_P$')
    plt.xlabel('time [s]')
    
    plt.figure(2)
    plt.grid(True)        
    plt.subplot(3, 1, 1)
    plt.title('Compressor inputs')
    plt.plot(time, compM)
    plt.grid(True)
    plt.ylabel(r'N$_M$')
    plt.ylim([0.910,0.925])
    #plt.ylim([0.912,0.922])
    plt.title('Compressor inputs')
    plt.subplot(3, 1, 2)
    plt.plot(time, compE)
    plt.grid(True)
    plt.ylabel(r'N$_E$')
    plt.ylim([0.875,0.890])
    #plt.ylim([0.878,0.888])
    plt.subplot(3, 1, 3)
    plt.plot(time, compP)
    plt.grid(True)
    plt.ylabel(r'N$_P$')
    plt.ylim([0.820,0.835])
    #plt.ylim([0.826,0.836])
    plt.xlabel('time [s]')

    plt.figure(3)
    plt.hold(True)
    plt.grid(True)
    plt.ylabel('Cooling')
    plt.title('Simulation') 
    plt.subplot(3, 1, 1)
    plt.title('Cooling')
    plt.plot(time, waterM)
    plt.grid(True)
    plt.ylabel('methane')
    plt.subplot(3, 1, 2)
    plt.plot(time, waterE,'--')
    plt.grid(True)
    plt.ylabel('ethane')
    plt.subplot(3, 1, 3)
    plt.plot(time, waterP)
    plt.grid(True)
    plt.ylabel('propane')
    
    plt.figure(4)
    plt.grid(True)
    plt.subplot(3, 1, 1)
    plt.title('Valve opening positions')
    plt.plot(time, valveM)
    plt.grid(True)
    plt.ylabel('Methane')
    plt.subplot(3, 1, 2)
    plt.plot(time, valveE)
    plt.grid(True)
    plt.ylabel('Ethane')
    plt.subplot(3, 1, 3)
    plt.plot(time, valveP)
    plt.grid(True)
    plt.ylabel('Propane')
    plt.xlabel('time [s]')

    plt.figure(5)
    plt.grid(True)
    plt.ylabel('Various')
    plt.title('Simulation')  
    plt.subplot(3, 1, 1)
    plt.ylabel('Various')
    plt.plot(time, tempL)
    plt.grid(True)
    plt.ylabel('LNGtemp')
    plt.subplot(3, 1, 2)
    plt.plot(time, Tamb)
    plt.grid(True)
    plt.ylabel('Tamb')
    plt.subplot(3, 1, 3)
    plt.plot(time, mm)
    plt.grid(True)
    plt.ylabel('flow')   
            
    plt.figure(6)
    plt.title('Random')
    plt.hold(True)
    plt.subplot(3, 1, 1)
    plt.title('Random')
    plt.plot(time, Press)
    #plt.plot(time_o, compM_o,'--')
    plt.ylabel('Pressure methane')
    plt.subplot(3, 1, 2)
    plt.hold(True)
    plt.plot(time, Tamb)
    plt.plot(time_o, Tamb_o,'--')
    plt.ylabel('Tamb')
    plt.grid(True)
    plt.subplot(3, 1, 3)
    plt.hold(True)
    plt.plot(time, Te)
    #plt.plot(time_o[:4], Tamb_o[:4])
    plt.ylabel('Ethane temp')
    plt.grid(True)

    plt.figure(8)
    plt.subplot(2, 1, 1)
    plt.grid(True)
    plt.hold(True)
    plt.title('Total work and LNG temperature')
    plt.plot(time, compW)
    plt.plot(timePI,workPI,'--')
    plt.legend(('MPC', 'PI controller'),loc=4)
    plt.ylabel('Total Work [MJ/h]')
    plt.subplot(2, 1, 2)
    plt.ylim([-150.4,-149.4])
    plt.grid(True)
    plt.hold(True)
    plt.plot(time, tempL)
    plt.plot(timePI,tempPI,'--')
    plt.legend(('MPC', 'PI controller'),loc=1)
    plt.ylabel('LNG temp [$\circ C$]')
    plt.xlabel('time [s]')

    plt.figure(14)
    plt.grid(True)
    plt.hold(True)
    plt.title('Total compressor work ')
    plt.plot(time[375:], compW[375:])
    plt.plot(timePI[403:],workPI[403:],'--')
    plt.legend(('MPC', 'PI controller'))
    plt.ylabel('Total Work [MJ/h]')
    plt.xlabel('time [s]')
    
    plt.figure(9)
    plt.subplot(1, 1, 1)
    plt.title('Cooling water temperature')
    plt.hold(True)
    plt.plot(time[:280], Tamb[:280])
    plt.ylim([4,11])
    plt.plot(time_o[:103], Tamb_o[:103],'--')
    plt.legend(('Actual temp', 'MPC sampled temp'),loc=4)
    plt.ylabel(r'T$_{amb}$ $[^\circ C]$' )
    plt.xlabel('time [s]')
    plt.grid(True)

    plt.figure(10)
    plt.subplot(3, 1, 1)
    plt.hold(True)
    plt.plot(time, compM)
    plt.plot(timePI, compMPI,'--')
    plt.grid(True)
    plt.ylabel(r'N$_M$')
    plt.ylim([0.910,0.925])
    plt.title('Compressor inputs')
    plt.legend(('MPC', 'PI controller'),loc=4)
    plt.subplot(3, 1, 2)
    plt.hold(True)
    plt.plot(time, compE)
    plt.plot(timePI, compEPI,'--')
    plt.grid(True)
    plt.ylabel(r'N$_E$')
    plt.ylim([0.875,0.890])
    plt.subplot(3, 1, 3)
    plt.hold(True)
    plt.plot(time, compP)
    plt.plot(timePI, compPPI,'--')
    plt.grid(True)
    plt.ylabel(r'N$_P$')
    plt.ylim([0.825,0.84])
    plt.xlabel('time [s]')

    plt.figure(11)
    plt.subplot(2, 1, 1)
    plt.hold(True)
    plt.plot(time, hexEL)
    plt.grid(True)
    plt.ylabel(r'temp [K]')
    plt.title('LNG temperatures')
    plt.subplot(2, 1, 2)
    plt.hold(True)
    plt.plot(time, hexPL)
    plt.grid(True)
    plt.ylabel(r'temp [K]')
    plt.xlabel('time [s]')
    
    plt.figure(12)
    plt.subplot(2, 1, 1)
    plt.grid(True)
    plt.hold(True)
    plt.title('Total work and LNG temperature')
    plt.plot(time, compW)
    plt.plot(timeR,workR,'--')
    plt.legend(('Standard MPC', 'Robust MPC'),loc=4)
    plt.ylabel('Total Work [MJ/h]')
    plt.subplot(2, 1, 2)
    plt.ylim([-150.6,-149.6])
    plt.grid(True)
    plt.hold(True)
    plt.plot(time, tempL)
    plt.plot(timeR,tempR,'--')
    plt.legend(('Standard MPC', 'Robust MPC'),loc=1)
    plt.ylabel('LNG temp [$^\circ C$]')
    plt.xlabel('time [s]')

    plt.figure(13)
    plt.subplot(3, 1, 1)
    plt.hold(True)
    plt.plot(time, compM)
    plt.plot(timeR, compMR,'--')
    plt.grid(True)
    plt.ylabel(r'N$_M$')
    plt.ylim([0.910,0.925])
    plt.title('Compressor inputs')
    plt.legend(('Standard MPC', 'Robust MPC'),loc=4)
    plt.subplot(3, 1, 2)
    plt.hold(True)
    plt.plot(time, compE)
    plt.plot(timeR, compER,'--')
    plt.grid(True)
    plt.ylabel(r'N$_E$')
    plt.ylim([0.875,0.890])
    plt.subplot(3, 1, 3)
    plt.hold(True)
    plt.plot(time, compP)
    plt.plot(timeR, compPR,'--')
    plt.grid(True)
    plt.ylabel(r'N$_P$')
    plt.ylim([0.825,0.84])
    plt.xlabel('time [s]')
    
    plt.show()


#Warm start options
def set_warm_start_options(solver, push=1e-4, mu_init=1e-1):    
    solver.set_solver_option('IPOPT', 'warm_start_init_point', 'yes')
    solver.set_solver_option('IPOPT', 'mu_init', mu_init)

    solver.set_solver_option('IPOPT', 'warm_start_bound_push', push)
    solver.set_solver_option('IPOPT', 'warm_start_mult_bound_push', push)
    solver.set_solver_option('IPOPT', 'warm_start_bound_frac', push)
    solver.set_solver_option('IPOPT', 'warm_start_slack_bound_frac', push)
    solver.set_solver_option('IPOPT', 'warm_start_slack_bound_push', push)

    
if __name__=="__main__":
    run_demo()
