import os.path
# Import numerical libraries
import numpy as N
import matplotlib.pyplot as plt

# Import the needed JModelica.org Python methods
from pymodelica import compile_fmu
from pyfmi import load_fmu
from pyjmi import transfer_optimization_problem, get_files_path
from pyjmi.optimization.casadi_collocation import BlockingFactors
from pyjmi.symbolic_elimination import BLTOptimizationProblem, EliminationOptions


def run_demo(with_plots=True):

    file_path = os.path.join(get_files_path(), "LNGtest.mop")
    
    init_fmu = compile_fmu("LNG.easy", file_path)
    
    init_model = load_fmu(init_fmu)

    ##Random input set B
    compM_B = 0.93
    compE_B = 0.87
    compP_B = 0.82
    mcondM_B = 0.05
    mcondE_B = 0.03
    mcondP_B = 0.25
    
    #set new input
    init_model.set('u1',compM_B)
    init_model.set('u2',compE_B)
    init_model.set('u3',compP_B)
    init_model.set('u4',mcondM_B)
    init_model.set('u5',mcondE_B)
    init_model.set('u6',mcondP_B)

    #Initialize for point B
    init_model.initialize()
    [HrecM_B,HrecE_B,TrecP_B] = init_model.get(['process.receiverM.Hrec', 'process.receiverE.Hrec','process.receiverP.Trec'])
    init_model.reset()

    #Random input setA
    compM_A = 0.94
    compE_A = 0.89
    compP_A = 0.86
    mcondM_A = 0.1
    mcondE_A = 0.1
    mcondP_A = 0.3
    init_model.set('u1',compM_A)
    init_model.set('u2',compE_A)
    init_model.set('u3',compP_A)
    init_model.set('u4',mcondM_A)
    init_model.set('u5',mcondE_A)
    init_model.set('u6',mcondP_A)

    
    # Solve the initialization problem for point A
    init_model.initialize()

    [HrecM_A,HrecE_A,TrecP_A] = init_model.get(['process.receiverM.Hrec', 'process.receiverE.Hrec','process.receiverP.Trec'])

    #simulation options
    opts = init_model.simulate_options()
    opts['initialize'] = False  
    opts['CVode_options']['atol'] = 1.0e-9
    opts['CVode_options']['rtol'] = 1.0e-7

    #set back inputs to point B (states are at point A)
    init_model.set('u1',compM_B)
    init_model.set('u2',compE_B)
    init_model.set('u3',compP_B)
    init_model.set('u4',mcondM_B)
    init_model.set('u5',mcondE_B)
    init_model.set('u6',mcondP_B)
    
    #simulate with input set B
    init_res = init_model.simulate(start_time=0., final_time=750.,options = opts)
   
    #Save states
    t = init_res['time']
    HM = init_res['process.receiverM.Hrec']*10
    HE = init_res['process.receiverE.Hrec']*10
    Tp = (init_res['process.receiverP.Trec']*100)-273

    #Plotting of initial trajectory from B to A
    if with_plots:
        plt.figure(0)
        plt.hold(True)
        plt.subplot(3, 1, 1)
        plt.plot(t, HM)
        
        plt.grid()
        plt.ylabel(r'Hrec$_M$ $\frac{J}{mol}$')
        plt.title('Dynamic states')

        plt.subplot(3, 1, 2)
        plt.plot(t, HE)
        plt.ylim([-1.048,-1.038])
        plt.grid()
        plt.ylabel(r'Hrec$_E$ $\frac{J}{mol}$')

        plt.subplot(3, 1, 3)
        plt.plot(t, Tp)
        plt.grid()
        plt.ylabel(r'Trec$_P$ [$^\circ C$]')
        plt.xlabel('time [s]')
        plt.show()
        
    #Create optimiation problem
    op = transfer_optimization_problem('LNG.opt', file_path)
    
     
    
    #set dynamic states in optimization problem
    op.set('HrecM',float(HrecM_A))
    op.set('HrecE',float(HrecE_A))
    op.set('TrecP',float(TrecP_A))

    
    #Options for dynamic problem
    opts = op.optimize_options()
    opts['n_e'] = 50 # Number of elements
    opts['init_traj'] = init_res
    opts['nominal_traj'] = init_res
    opts['IPOPT_options']['tol'] = 1e-7
    opts['verbosity'] = 3

    #Blockingfactors
    # Set blocking factors
    factors = {'u1': opts['n_e'] / 1 * [1],
               'u2': opts['n_e'] / 1 * [1],
               'u3': opts['n_e'] / 1 * [1],
               'u4': opts['n_e'] / 1 * [1],
               'u5': opts['n_e'] / 1 * [1],
               'u6': opts['n_e'] / 1 * [1]}
    #Penalties on inputs can be used (not used here)
    du_quad_pen = {'u1': 5,
                   'u2':5,
                   'u4':5,
                   'u4':1,
                   'u5':1,
                   'u6':1}
    
    bf = BlockingFactors(factors)
    opts['blocking_factors'] = bf

    # Solve the optimal control problem)
    res = op.optimize(options=opts)
    
    # Extract variable profiles
    hrec = res['process.receiverM.Hrec']*10
    erec = res['process.receiverE.Hrec']*10
    Tprop = (res['process.receiverP.Trec']*100)- 273
    time_res = res['time']
    
    
    # Plot the results
    if with_plots:
        plt.figure(1)
        plt.hold(True)
        plt.subplot(3, 1, 1)
        plt.plot(time_res, hrec)
        
        plt.grid()
        plt.ylabel(r'Hrec$_M [\frac{J}{Mol}]')
        plt.title('Dynamic states')

        plt.subplot(3, 1, 2)
        plt.plot(time_res, erec)
        plt.ylim([-0.10442,-0.104])
        plt.grid()
        plt.ylabel(r'Hrec$_E$ [\frac{J}{Mol}]')

        plt.subplot(3, 1, 3)
        plt.plot(time_res, Tprop)
        plt.grid()
        plt.ylabel('Trec [K]')
        plt.xlabel('time')
        
        valveM = res['process.valveM.x'][:100]
        valveE = res['process.valveE.x'][:100]
        valveP = res['process.valveP.x'][:100]
        TLNG = (res['process.LNGsink.inlet.T'][:100])*100 -273
        
        
        plt.figure(2)
        plt.hold(True)
        plt.subplot(4, 1, 1)
        plt.plot(time_res[:100], valveM)
        plt.grid()
        #plt.ylim([0.5,0.6])
        plt.ylabel('Methane ')
        plt.title('Valve opening positions')

        plt.subplot(4, 1, 2)
        plt.plot(time_res[:100], valveE)
        #plt.ylim([0.5,0.6])
        plt.grid()
        plt.ylabel('Ethane')

        plt.subplot(4, 1, 3)
        plt.plot(time_res[:100], valveP)
        #plt.ylim([0.5185,0.52])
        plt.grid()
        plt.ylabel('Propane')
        

        plt.subplot(4, 1, 4)
        plt.plot(time_res[:100], TLNG)
        plt.ylim([-149.4,-150.6])
        plt.grid()
        plt.ylabel('LNG temp [$^\circ C$]')
        plt.xlabel('time [s]')

        N_MM = res['process.compM.N_M']
        N_ME = res['process.compE.N_M']
        N_MP = res['process.compP.N_M']
        tot = res['process.compM.Wcomp']+res['process.compE.Wcomp']+res['process.compP.Wcomp']
        tot2 = (res['process.condM.mcond'] + res['process.condE.mcond'] +res['process.condP.mcond'])*0.001  
        
        plt.figure(3)
        plt.hold(True)
        plt.subplot(4, 1, 1)
        plt.plot(time_res, N_MM)
        plt.grid()
        plt.ylim([0.91,0.94])
        plt.ylabel(r'$N_M$')
        plt.title('Compressors rotation speed inputs, and total work')

        plt.subplot(4, 1, 2)
        plt.plot(time_res, N_ME)
        plt.ylim([0.86,0.89])
        plt.grid()
        plt.ylabel(r'$N_E$')

        plt.subplot(4, 1, 3)
        plt.plot(time_res, N_MP)
        plt.ylim([0.82,0.85])
        plt.grid()
        plt.ylabel(r'$N_P$')

        plt.subplot(4, 1, 4)
        plt.plot(time_res, tot*100)
        #plt.plot(time_res, tot2,'--')
        
        plt.grid()
        plt.ylabel('Total work [MJ/h]')
        plt.xlabel('time [s]')

        condM = res['process.condE.mcond']
        condET = (res['process.condE.outlet.T']*100)-273
        condETT = (res['process.condE.Tcond']*100)-273
        
        TLNG = res['process.LNGsink.inlet.T']
        
        plt.figure(4)
        plt.hold(True)
        plt.subplot(2, 1, 1)
        plt.plot(time_res, condM)
        #plt.ylim([0,1.1])
        plt.grid()
        plt.ylabel(r'mcond$_E$')
        plt.title('Ethane condenser cooling water flow, and outlet temperatures')

        plt.subplot(2, 1, 2)
        plt.plot(time_res, condET)
        plt.plot(time_res,condETT)
        plt.legend(('Ethane outlet temp', 'Water outlet temp'),loc=7)
        #plt.ylim([0,1.1])
        plt.grid()
        plt.ylabel('Temp [$^\circ C$]')
        plt.xlabel('time [s]')


        MT = res['process.valveM.outlet.T']
        ET = res['process.valveE.outlet.T']
        PT = res['process.valveP.outlet.T']
        

        plt.figure(5)
        plt.hold(True)
        plt.subplot(3, 1, 1)
        plt.plot(time_res, MT)
        
        plt.grid()
        plt.ylabel('M')
        plt.title('Valve outlet T')

        plt.subplot(3, 1, 2)
        plt.plot(time_res, ET)
        
        plt.grid()
        plt.ylabel('E')
        

        plt.subplot(3, 1, 3)
        plt.plot(time_res, PT)
        
        plt.grid()
        plt.ylabel('P')
        plt.xlabel('time')


        MP = res['process.valveM.outlet.P']
        EP = res['process.valveE.outlet.P']
        PP = res['process.valveP.outlet.P']
        

        plt.figure(6)
        plt.hold(True)
        plt.subplot(3, 1, 1)
        plt.plot(time_res, MP)
        
        plt.grid()
        plt.ylabel('M')
        plt.title('Valve outlet P')

        plt.subplot(3, 1, 2)
        plt.plot(time_res, EP)
        
        plt.grid()
        plt.ylabel('E')
        

        plt.subplot(3, 1, 3)
        plt.plot(time_res, PP)
        
        plt.grid()
        plt.ylabel('P')
        plt.xlabel('time')

        MM = res['process.valveM.outlet.m']*-1
        EM = res['process.valveE.outlet.m'] *-1
        PM = res['process.valveP.outlet.m']*-1
        

        plt.figure(7)
        plt.hold(True)
        plt.subplot(3, 1, 1)
        plt.plot(time_res, MM)
        
        plt.grid()
        plt.ylabel(r'Methane')
        plt.title(r'Flow rates in each cycle $\frac{kmol}{h}* 10^4$')

        plt.subplot(3, 1, 2)
        plt.plot(time_res, EM)
        
        plt.grid()
        plt.ylabel(r'Ethane')
        

        plt.subplot(3, 1, 3)
        plt.plot(time_res, PM)
        
        plt.grid()
        plt.ylabel(r'Propane')
        plt.xlabel('time [s]')
        plt.show()

        

if __name__=="__main__":
    run_demo()


    
