import os.path
import csv
# Import numerical libraries
import numpy as N
import matplotlib.pyplot as plt

# Import the needed JModelica.org Python methods
from pymodelica import compile_fmu
from pyfmi import load_fmu
from pyjmi import transfer_optimization_problem, get_files_path
from pyjmi.optimization.casadi_collocation import BlockingFactors
from pyjmi.symbolic_elimination import BLTOptimizationProblem, EliminationOptions


def run_demo(with_plots=True):

    file_path = os.path.join(get_files_path(), "LNGrobust.mop")
    
    init_fmu = compile_fmu("LNG.easy", file_path)
    
    init_model = load_fmu(init_fmu)

    #Random point A
    #compM_A = 0.94
    #compE_A = 0.89
    #compP_A = 0.86
    #mcondM_A = 0.1
    #mcondE_A = 0.1
    #mcondP_A = 0.3
    #Optimal point A
    compM_A = 0.921778
    compE_A = 0.883772
    compP_A = 0.830676
    mcondM_A = 0.134422
    mcondE_A = 0.112109
    mcondP_A = 0.999998
    #Everything now done for all three cases
    init_model.set('u1',compM_A)
    init_model.set('u2',compE_A)
    init_model.set('u3',compP_A)
    init_model.set('u4',mcondM_A)
    init_model.set('u5',mcondE_A)
    init_model.set('u6',mcondP_A)
    init_model.set('u1_l',compM_A)
    init_model.set('u2_l',compE_A)
    init_model.set('u3_l',compP_A)
    init_model.set('u4_l',mcondM_A)
    init_model.set('u5_l',mcondE_A)
    init_model.set('u6_l',mcondP_A)
    init_model.set('u1_u',compM_A)
    init_model.set('u2_u',compE_A)
    init_model.set('u3_u',compP_A)
    init_model.set('u4_u',mcondM_A)
    init_model.set('u5_u',mcondE_A)
    init_model.set('u6_u',mcondP_A)

    
    # Solve the initialization problem A using FMI
    init_model.initialize()
    #Save point A
    [HrecM_A,HrecE_A,TrecP_A] = init_model.get(['process.receiverM.Hrec', 'process.receiverE.Hrec','process.receiverP.Trec'])
    
    #simulation options
    opts = init_model.simulate_options()
    opts['initialize'] = False  
    opts['CVode_options']['atol'] = 1.0e-9
    opts['CVode_options']['rtol'] = 1.0e-7

    #Initial trajectory guess input B
    compM_B = 0.93
    compE_B = 0.87
    compP_B = 0.82
    mcondM_B = 0.05
    mcondE_B = 0.03
    mcondP_B = 0.25
    #Set input B
    init_model.set('u1',compM_B)
    init_model.set('u2',compE_B)
    init_model.set('u3',compP_B)
    init_model.set('u4',mcondM_B)
    init_model.set('u5',mcondE_B)
    init_model.set('u6',mcondP_B)
    init_model.set('u1_l',compM_B)
    init_model.set('u2_l',compE_B)
    init_model.set('u3_l',compP_B)
    init_model.set('u4_l',mcondM_B)
    init_model.set('u5_l',mcondE_B)
    init_model.set('u6_l',mcondP_B)
    init_model.set('u1_u',compM_B)
    init_model.set('u2_u',compE_B)
    init_model.set('u3_u',compP_B)
    init_model.set('u4_u',mcondM_B)
    init_model.set('u5_u',mcondE_B)
    init_model.set('u6_u',mcondP_B)

    #Upper and lower case for cooling water temperature
    init_model.set('Tamb_set_l',2.77)
    init_model.set('Tamb_set_u',2.79)
    #Simulate with input B for initial trajectory
    init_res = init_model.simulate(start_time=0., final_time=700.,options = opts)

    #Optimization problem
    op = transfer_optimization_problem('LNG.opt', file_path)
    
    #Simulation model
    sim_fmu = compile_fmu("LNG.easysim", file_path)
    sim_model = load_fmu(sim_fmu)
    
    #Initial simulation conditions
    sim_model.set('HrecM',HrecM_A)
    sim_model.set('HrecE',HrecE_A)
    sim_model.set('TrecP',TrecP_A) 

    #options
    opts_S = init_model.simulate_options()
    opts_S['initialize'] = False  
    opts_S['CVode_options']['atol'] = 1.0e-9
    opts_S['CVode_options']['rtol'] = 1.0e-7

    sample_time = 20
    horizon = 700
    n_e = horizon/sample_time
    final_t =760

    num_input = final_t/sample_time

    #set Initial receiver states
    #All cases have same states
    op.set('HrecM',float(HrecM_A))
    op.set('HrecE',float(HrecE_A))
    op.set('TrecP',float(TrecP_A))
    
    op.set('HrecM_l',float(HrecM_A))
    op.set('HrecE_l',float(HrecE_A))
    op.set('TrecP_l',float(TrecP_A))

    op.set('HrecM_u',float(HrecM_A))
    op.set('HrecE_u',float(HrecE_A))
    op.set('TrecP_u',float(TrecP_A))
    
    #Upper and lower have different temperature
    op.set('Tamb_l',float(2.77))
    op.set('Tamb_set_l',float(2.77))
    op.set('Tamb_u',float(2.79))
    op.set('Tamb_set_u',float(2.79))
    
    #options
    opts = op.optimize_options()
    opts['n_e'] = n_e # Number of elements
    
    opts['init_traj'] = init_res
    opts['nominal_traj'] = init_res
    opts['IPOPT_options']['tol'] = 1e-7
    #opts['verbosity'] = 3

    #Blockingfactors
    # Set blocking factors
    factors = {'u1': opts['n_e'] / 1 * [1],
               'u2': opts['n_e'] / 1 * [1],
               'u3': opts['n_e'] / 1 * [1],
               'u4': opts['n_e'] / 1 * [1],
               'u5': opts['n_e'] / 1 * [1],
               'u6': opts['n_e'] / 1 * [1],
               'u1_l': opts['n_e'] / 1 * [1],
               'u2_l': opts['n_e'] / 1 * [1],
               'u3_l': opts['n_e'] / 1 * [1],
               'u4_l': opts['n_e'] / 1 * [1],
               'u5_l': opts['n_e'] / 1 * [1],
               'u6_l': opts['n_e'] / 1 * [1],
               'u1_u': opts['n_e'] / 1 * [1],
               'u2_u': opts['n_e'] / 1 * [1],
               'u3_u': opts['n_e'] / 1 * [1],
               'u4_u': opts['n_e'] / 1 * [1],
               'u5_u': opts['n_e'] / 1 * [1],
               'u6_u': opts['n_e'] / 1 * [1]}
    #quadratic penalties can be used(not used here)
    du_quad_pen = {'u1':10,
                   'u2':10,
                   'u4':10,
                   'u4':10,
                   'u5':10,
                   'u6':10,
                   'u1_l':10,
                   'u2_l':10,
                   'u4_l':10,
                   'u4_l':10,
                   'u5_l':10,
                   'u6_l':10,
                   'u1_u':10,
                   'u2_u':10,
                   'u4_u':10,
                   'u4_u':10,
                   'u5_u':10,
                   'u6_u':10}
    
    bf = BlockingFactors(factors)
    opts['blocking_factors'] = bf

    #Lists for logging simulation variables
    HM = []
    HE = []
    TP = []
    compM = []
    compM_o = []
    compE = []
    compP = []
    compW = []
    waterM = []
    waterE = []
    waterP = []
    valveM = []
    valveE = []
    valveP = []
    tempL = []
    Tamb = []
    Te = []
    Press = []
    mm =[]
    time = []
    time_o = []
    for n in range(num_input):        
        if n == 0:
            sim_model.initialize()
            solver = op.prepare_optimization(options=opts)
            #Option for IPOPT to handle larger NLP
            solver.set_solver_option('IPOPT', 'mumps_mem_percent', 50)

        if n == 1:
            solver.set_warm_start(True)
            set_warm_start_options(solver)
        if n ==6:
            sim_model.set('Tamb_set',2.7875)
            
        #Set time in solver
        solver.collocator.t0 = n*sample_time
        solver.collocator.tf = n*sample_time+horizon
    
        #Solve OCP    
        res = solver.optimize()
        #Random plotting logging
        testm1 = res['process.compM.N_M'][:76]
        testm2 = res['process_l.compM.N_M'][:76]
        testm3 = res['process_u.compM.N_M'][:76]
        teste1 = res['process.compE.N_M'][:76]
        teste2 = res['process_l.compE.N_M'][:76]
        teste3 = res['process_u.compE.N_M'][:76]
        testp1 = res['process.compP.N_M'][:76]
        testp2 = res['process_l.compP.N_M'][:76]
        testp3 = res['process_u.compP.N_M'][:76]
        
        testtime = res['time'][:76]


        if n== 1:
            plt.figure(7)
            plt.subplot(3, 1, 1)
            plt.grid(True)
            plt.hold(True)
            plt.ylabel(r'N$_M$')
            plt.ylim([0.910,0.925])
            plt.title('Compressor inputs')
            plt.ylim([0.910,0.925])
            plt.plot(testtime, testm1)
            plt.plot(testtime, testm2)
            plt.plot(testtime, testm3)
            plt.subplot(3, 1, 2)
            plt.grid(True)
            plt.ylabel(r'N$_E$')
            plt.ylim([0.875,0.890])
            plt.title('test')
            plt.plot(testtime, teste1)
            plt.plot(testtime, teste2)
            plt.plot(testtime, teste3)
            plt.legend((r'$T_{amb}$',r'$T_{amb,l}$',r'$T_{amb,u}$'),loc =1)
            plt.subplot(3, 1, 3)
            plt.grid(True)
            plt.ylabel(r'N$_P$')
            plt.ylim([0.820,0.835])
            plt.plot(testtime, testp1)
            plt.plot(testtime, testp2)
            plt.plot(testtime, testp3)
            plt.xlabel('time [s]')
            plt.show()
        
        #Get opt input
        opt_input = res.get_opt_input()
        
        #Simulate for 1 sample time with opt input
        sim_res = sim_model.simulate(start_time= n*sample_time, final_time=(n+1)*sample_time,
                             input=(opt_input), options=opts_S)
        
        #Log changes to states and inputs for plotting 
        HM.extend(sim_res['process.receiverM.Hrec'])
        HE.extend(sim_res['process.receiverE.Hrec'])
        TP.extend(sim_res['process.receiverP.Trec'])
        
        compM.extend(sim_res['process.compM.N_M'])
        compE.extend(sim_res['process.compE.N_M'])
        compP.extend(sim_res['process.compP.N_M'])
        compW.extend(sim_res['process.compM.Wcomp']+sim_res['process.compE.Wcomp']+sim_res['process.compP.Wcomp'])      
        
        waterM.extend(sim_res['process.condM.mcond'])
        waterE.extend(sim_res['process.condE.mcond'])
        waterP.extend(sim_res['process.condP.mcond'])
        
        valveM.extend(sim_res['process.valveM.x'])
        valveE.extend(sim_res['process.valveE.x'])
        valveP.extend(sim_res['process.valveP.x'])
        
        tempL.extend(sim_res['process.LNGsink.inlet.T']*100)
        Tamb.extend(sim_res['process.Tamb'])
        Te.extend(sim_res['process.condE.outlet.T'])
        Press.extend(sim_res['process.compM.outlet.P'])
        mm.extend(sim_res['process.valveM.outlet.m'])
        time.extend(sim_res['time'])

        #generate noise
        num1 = N.random.normal(1,0.005)
        num2 = N.random.normal(1,0.005)
        num3 = N.random.normal(1,0.005)
        
        #Extract end simulation states
        HrecM = sim_res['process.receiverM.Hrec'][-1]
        HrecE = sim_res['process.receiverE.Hrec'][-1]
        TrecP = sim_res['process.receiverP.Trec'][-1]
        
        
        Tambient = sim_res['process.Tamb'][-1]
        LNGtemp = sim_res['process.LNGtemp'][-1]
        LNGflow = sim_res['process.LNGflow'][-1]
       

        alphaM = sim_res['process.alphaM'][-1]
        alphaE = sim_res['process.alphaE'][-1]
        alphaP = sim_res['process.alphaP'][-1]
        
        
        #Create new op problem by resetting states
        #set new states
        
        solver.set('HrecM',float(HrecM))
        solver.set('HrecE',float(HrecE))
        solver.set('TrecP',float(TrecP))

        #solver.set('Tamb',float(Tambient)) unnoticed change
        solver.set('LNGtemp',float(LNGtemp))
        solver.set('LNGflow',float(LNGflow))

        solver.set('alphaM',float(alphaM))
        solver.set('alphaE',float(alphaE))
        solver.set('alphaP',float(alphaP))

        solver.set('HrecM_l',float(HrecM))
        solver.set('HrecE_l',float(HrecE))
        solver.set('TrecP_l',float(TrecP))

        #solver.set('Tamb_l',float(Tambient)) unnoticed change
        solver.set('LNGtemp_l',float(LNGtemp))
        solver.set('LNGflow_l',float(LNGflow))

        solver.set('alphaM_l',float(alphaM))
        solver.set('alphaE_l',float(alphaE))
        solver.set('alphaP_l',float(alphaP))

        solver.set('HrecM_u',float(HrecM))
        solver.set('HrecE_u',float(HrecE))
        solver.set('TrecP_u',float(TrecP))

        #solver.set('Tamb_u',float(Tambient)) unnoticed change
        solver.set('LNGtemp_u',float(LNGtemp))
        solver.set('LNGflow_u',float(LNGflow))

        solver.set('alphaM_u',float(alphaM))
        solver.set('alphaE_u',float(alphaE))
        solver.set('alphaP_u',float(alphaP))


    #Plotting
    

    plt.figure(1)
    plt.grid(True)
    plt.ylabel('Receivers')
    plt.title('Simulation')
    plt.hold(True)
    plt.subplot(3, 1, 1)
    plt.title('Receivers')
    plt.plot(time, HM)
    plt.ylabel('Methane')
       
    plt.subplot(3, 1, 2)
    plt.plot(time, HE)
    plt.grid(True)
    plt.ylabel('Ethane')
    
    plt.subplot(3, 1, 3)
    plt.plot(time, TP)
    plt.grid(True)
    plt.ylabel('Propane')


    plt.figure(2)
    plt.grid(True)
    plt.ylabel('Compressors')
    plt.title('Simulation')
            
    plt.subplot(3, 1, 1)
    plt.plot(time, compM)
    plt.grid(True)
    plt.ylabel('methane')

    plt.subplot(3, 1, 2)
    plt.plot(time, compE)
    plt.grid(True)
    plt.ylabel('ethane')
    plt.subplot(3, 1, 3)
    plt.plot(time, compP)
    plt.grid(True)
    plt.ylabel('Speed')

    plt.figure(3)
    plt.hold(True)
    plt.grid(True)
    plt.ylabel('Cooling')
    plt.title('Simulation')
            
    plt.subplot(3, 1, 1)
    plt.title('Cooling')
    plt.plot(time, waterM)
    plt.grid(True)
    plt.ylabel('methane')

    plt.subplot(3, 1, 1)
    plt.plot(time, waterE,'--')
    plt.grid(True)
    plt.ylabel('ethane')
    plt.subplot(3, 1, 3)
    plt.plot(time, waterP)
    plt.grid(True)
    plt.ylabel('propane')

    plt.figure(4)
    plt.grid(True)
    plt.ylabel('valves')
    plt.title('Simulation')
            
    plt.subplot(3, 1, 1)
    plt.ylabel('valves')
    plt.plot(time, valveM)
    plt.grid(True)
    plt.ylabel('methane')

    plt.subplot(3, 1, 2)
    plt.plot(time, valveE)
    plt.grid(True)
    plt.ylabel('ethane')
    plt.subplot(3, 1, 3)
    plt.plot(time, valveP)
    plt.grid(True)
    plt.ylabel('propane')

    plt.figure(5)
    plt.grid(True)
    plt.ylabel('Various')
    plt.title('Simulation')
            
    plt.subplot(1, 1, 1)
    plt.plot(time, tempL)
    plt.grid(True)
    plt.ylim([-150.6,-149.4])
    plt.title('LNG outlet temperature')
    plt.ylabel(r'Temp [$\circ C$]')
    plt.xlabel('time [s]')

            
            
    plt.figure(6)
    plt.title('Random')
    plt.hold(True)
    plt.subplot(3, 1, 1)
    plt.title('Random')
    plt.plot(time, Press)
    #plt.plot(time_o, compM_o,'--')
    plt.ylabel('Pressure methane')
            
    plt.subplot(3, 1, 2)
    plt.hold(True)
    plt.plot(time, Tamb)
    #plt.plot(time_o[:4], Tamb_o[:4],'--')
    plt.ylabel('Tamb')
    plt.grid(True)

    plt.subplot(3, 1, 3)
    plt.hold(True)
    plt.plot(time, Te)
    #plt.plot(time_o[:4], Tamb_o[:4])
    plt.ylabel('Ethane temp')
    plt.grid(True)


    plt.figure(8)
    
    plt.subplot(2, 1, 1)
    plt.grid(True)
    plt.hold(True)
    plt.title('Total work and LNG temperature')
    plt.plot(time, compW)
    #plt.plot(timePI,workPI,'--')
    plt.legend(('MPC', 'PI controller'),loc=4)
    plt.ylabel('Total Work')
    plt.subplot(2, 1, 2)
    plt.ylim([122.6,123.6])
    plt.grid(True)
    plt.hold(True)
    plt.plot(time, tempL)
    #plt.plot(timePI,tempPI,'--')
    plt.legend(('MPC', 'PI controller'),loc=1)
    plt.ylabel('LNG temp [K]')
    plt.xlabel('time [s]')

    #If solution is to be logged
    #with open('testfile.csv', 'wb') as myfile:
        #wr = csv.writer(myfile, quoting=csv.QUOTE_ALL)
        #wr.writerow(time)
        #wr.writerow(compW)
        #wr.writerow(tempL)
        #wr.writerow(compM)
        #wr.writerow(compE)
        #wr.writerow(compP)
    
    plt.show()


def set_warm_start_options(solver, push=1e-4, mu_init=1e-1):    
    solver.set_solver_option('IPOPT', 'warm_start_init_point', 'yes')
    solver.set_solver_option('IPOPT', 'mu_init', mu_init)

    solver.set_solver_option('IPOPT', 'warm_start_bound_push', push)
    solver.set_solver_option('IPOPT', 'warm_start_mult_bound_push', push)
    solver.set_solver_option('IPOPT', 'warm_start_bound_frac', push)
    solver.set_solver_option('IPOPT', 'warm_start_slack_bound_frac', push)
    solver.set_solver_option('IPOPT', 'warm_start_slack_bound_push', push)

    
if __name__=="__main__":
    run_demo()
